/*
 * This file is part of Maliit plugins
 *
 * Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 *
 * Contact: Mohammad Anwari <Mohammad.Anwari@nokia.com>
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list
 * of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list
 * of conditions and the following disclaimer in the documentation and/or other materials
 * provided with the distribution.
 * Neither the name of Nokia Corporation nor the names of its contributors may be
 * used to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

import QtQuick 2.0
import org.nemomobile 1.0

import "Screen.js" as ScreenSettings

import QtQuick.Window 2.0
import Ubuntu.Components 0.1

Item {
    id: canvas
    transformOrigin: Item.Center
    width: MInputMethodQuick.screenWidth
    height: units.gu(ScreenSettings.LANDSCAPE_SURFACE_HEIGHT);

    property int visibleHeight: 0

    property bool shown: false
    property int orientation: Screen.orientation;

    Item {
        id: root
        transformOrigin: Item.Center

        anchors.fill: parent

        state: "HIDDEN"

        KeyboardBase {
            id: keyboard

            x:0
            y:0
            width: parent.width
            height: parent.height

            Rectangle {
                id: spacer
                anchors.top: parent.top
                width: parent.width
                height: canvas.height - canvas.visibleHeight
                color: "transparent"
            }

            EnglishLandscape {
                id: vkb_landscape
                visible: keyboard.layout == vkb_landscape
                height: units.gu(ScreenSettings.LANDSCAPE_KEYBOARD_VISIBLE_HEIGHT)
                anchors.top: spacer.bottom
                anchors.horizontalCenter: parent.horizontalCenter
            }

            EnglishPortrait {
                id: vkb_portrait
                height: units.gu(ScreenSettings.PORTRAIT_KEYBOARD_VISIBLE_HEIGHT);
                visible: keyboard.layout == vkb_portrait
                anchors.top: spacer.bottom
                anchors.horizontalCenter: parent.horizontalCenter
            }
        }

        Component.onCompleted: {
            MInputMethodQuick.actionKeyOverride.setDefaultIcon("icon-m-input-methods-enter.svg")
            MInputMethodQuick.actionKeyOverride.setDefaultLabel("")
        }

        states: [
            State {
                name: "SHOWN"
                PropertyChanges { target: canvas; y: 0; }
                when: canvas.shown === true
            },

            State {
                name: "HIDDEN"
                PropertyChanges { target: canvas; y: height; }
                onCompleted: {
                    keyboard.y = 0;
                    MInputMethodQuick.hideWindow();
                    vkb_portrait.resetUI();
                    vkb_landscape.resetUI();
                }
                when: canvas.shown === false
            }
        ]
        transitions: Transition {
            PropertyAnimation { target: canvas; properties: "y"; easing.type: Easing.InOutQuad }
        }
    }

    focus: true

    function updateIMArea() {
        MInputMethodQuick.setInputMethodArea(Qt.rect(0, (Screen.height - canvas.visibleHeight), Screen.width, canvas.visibleHeight));
    }

    states: [
        State {
            name: "landscape"
            when: orientation == Qt.LandscapeOrientation || orientation == Qt.InvertedLandscapeOrientation

            PropertyChanges {
                target: canvas
                height:        units.gu(ScreenSettings.LANDSCAPE_SURFACE_HEIGHT);
                visibleHeight: units.gu(ScreenSettings.LANDSCAPE_KEYBOARD_VISIBLE_HEIGHT)
            }

            PropertyChanges {
                target: keyboard
                layout: vkb_landscape
            }

            onCompleted:  updateIMArea();
        },

        State {
            name: "portrait"
            when: orientation == Qt.PortraitOrientation || orientation == Qt.InvertedPortraitOrientation

            PropertyChanges {
                target: canvas
                height:        units.gu(ScreenSettings.PORTRAIT_SURFACE_HEIGHT);
                visibleHeight: units.gu(ScreenSettings.PORTRAIT_KEYBOARD_VISIBLE_HEIGHT)
            }

            PropertyChanges {
                target: keyboard
                layout: vkb_portrait
            }

            onCompleted:  updateIMArea();
        }
    ]
}

