/*
 * This file is part of Maliit Plugins
 *
 * Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 *
 * Contact: Mohammad Anwari <Mohammad.Anwari@nokia.com>
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list
 * of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list
 * of conditions and the following disclaimer in the documentation and/or other materials
 * provided with the distribution.
 * Neither the name of Nokia Corporation nor the names of its contributors may be
 * used to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

import QtQuick 2.0
import "KeyboardUiConstants.js" as UI
import Ubuntu.Components 0.1

KeyArea {
    id: keyArea

    anchors.topMargin: units.gu(UI.portraitTopMargin)
    spacing: units.gu(UI.landscapeSpacing)

    property int columns: Math.max(row1.length, row2.length, row3.length)
    property int keyWidth: units.gu(UI.landscapeWidth)
    property int keyHeight: units.gu(UI.landscapeHeight)
    property int keyMargin: UI.landscapeMargin

    Row { //Row 1
        anchors.horizontalCenter: parent.horizontalCenter
        spacing: keyMargin
        Repeater {
            model: row1
            CharacterKey {
                width: keyWidth; height: keyHeight
                caption: row1[index][0]
                captionShifted: row1[index][0].toUpperCase()
                symView: row1[index][1]
                symView2: row1[index][2]
            }
        }
    } //end Row1

    Row { //Row 2
        anchors.horizontalCenter: parent.horizontalCenter
        spacing: keyMargin
        Repeater {
            model: row2
            CharacterKey {
                width: keyWidth; height: keyHeight
                caption: row2[index][0]
                captionShifted: row2[index][0].toUpperCase()
                symView: row2[index][1]
                symView2: row2[index][2]
            }
        }
    } //end Row2

    Row { //Row 3
        anchors.horizontalCenter: parent.horizontalCenter
        spacing: keyMargin

        ShiftKey {
            state: {
                if (inSymView) {
                    return "SYMBOLS"
                } else if (isShifted) {
                    if (isShiftLocked)
                        return "CAPSLOCK"; 
                    else
                        return "SHIFT";
                }
                return ""
            }
                    
            width: units.gu(UI.LANDSCAPE_KEYUPPER_WIDTH); height: keyHeight
            landscape: true
            caption: inSymView ? (inSymView2 ? "2/2" : "1/2") : ""
            opacity: (mouseArea.containsMouse || (isShiftLocked && (!inSymView))) ? 0.6 : 1
            onClickedPass: {
                if (inSymView) {
                    inSymView2 = !inSymView2
                } else {
                    isShifted = (!isShifted)
                    isShiftLocked = false
                }
            }
            onPressedAndHoldPass: {
                if (!inSymView) {
                    isShifted = true
                    isShiftLocked = true
                }
            }
        }

        Row {
            spacing: keyMargin
            Repeater {
                model: row3
                CharacterKey {
                    width: keyWidth; height: keyHeight
                    caption: row3[index][0]
                    captionShifted: row3[index][0].toUpperCase()
                    symView: row3[index][1]
                    symView2: row3[index][2]
                }
            }
        }

        FunctionKey {
            width: units.gu(UI.LANDSCAPE_BACKSPACE_WIDTH); height: keyHeight
            landscape: true
            repeat: true
            icon: "icon_backspace.png"
            onClickedPass: MInputMethodQuick.sendCommit("\b");
        }
    } //end Row3

    Row { //Row 4
        anchors.horizontalCenter: parent.horizontalCenter
        spacing: keyMargin
        FunctionKey {
            width: units.gu(UI.LANDSCAPE_KEY123_WIDTH); height: keyHeight
            landscape: true
            caption: inSymView ? "ABC" : "?123"
            onClickedPass: inSymView = (!inSymView)
        }

        Row {
            spacing: keyMargin
            CharacterKey { caption: ","; captionShifted: ","; width: units.gu(UI.landscapeWidth); height: keyHeight; }
            CharacterKey { caption: " "; captionShifted: " "; width: units.gu(UI.LANDSCAPE_SPACE_WIDTH); height: keyHeight; }
            CharacterKey { caption: "."; captionShifted: "."; width: units.gu(UI.landscapeWidth); height: keyHeight; }
        }

        FunctionKey {
            width: units.gu(UI.LANDSCAPE_ENTER_WIDTH); height: keyHeight
            icon: "icon_enter.png"
            repeat: true
            caption: MInputMethodQuick.actionKeyOverride.label
            onClickedPass: MInputMethodQuick.activateActionKey()
        }
    } //end Row4
}

