/*
 * This file is part of Maliit plugins
 *
 * Copyright (C) Jakub Pavelek <jpavelek@live.com>
 * Copyright (C) 2012 John Brooks <john.brooks@dereferenced.net>
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list
 * of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list
 * of conditions and the following disclaimer in the documentation and/or other materials
 * provided with the distribution.
 * Neither the name of Nokia Corporation nor the names of its contributors may be
 * used to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

import QtQuick 2.0
import "KeyboardUiConstants.js" as UI

KeyArea {
    id: keyArea

    anchors.topMargin: units.gu(UI.portraitTopMargin)
    spacing: 16

    property int columns: Math.max(row1.length, row2.length, row3.length)
    property int keyWidth: (columns == 11)
                           ? units.gu(UI.portraitWidthNarrow)
                           : units.gu(UI.portraitWidth)
    property int keyHeight: units.gu(UI.portraitHeight)
    property int keyMargin: (columns == 11)
                            ? units.gu(UI.portraitMarginNarrow)
                            : units.gu(UI.portraitMargin)

    Row { //Row 1
        anchors.horizontalCenter: parent.horizontalCenter
        spacing: keyMargin
        Repeater {
            model: row1
            CharacterKey {
                width: keyWidth; height: keyHeight
                caption: row1[index][0]
                captionShifted: row1[index][0].toUpperCase()
                symView: row1[index][1]
                symView2: row1[index][2]
            }
        }
    } //end Row1

    Row { //Row 2
        anchors.horizontalCenter: parent.horizontalCenter

        spacing: keyMargin
        Repeater {
            model: row2
            CharacterKey {
                width: keyWidth; height: keyHeight
                caption: row2[index][0]
                captionShifted: row2[index][0].toUpperCase()
                symView: row2[index][1]
                symView2: row2[index][2]
            }
        }
    }

    Row { //Row 3
        anchors.horizontalCenter: parent.horizontalCenter
        spacing: (columns == 11) ? 32 : 16

        ShiftKey {
            state: {
                if (inSymView || inSymView2)
                    return "SYMBOLS"
                else if (isShifted) {
                    if (isShiftLocked)
                        return "CAPSLOCK";
                    else
                        return "SHIFT";
                }
                return ""
            }

            width: units.gu(UI.PORTRAIT_SHIFT_WIDTH); height: keyHeight
            caption: inSymView ? (inSymView2 ? "2/2" : "1/2") : ""
            opacity: (mouseArea.containsMouse || (isShiftLocked && (!inSymView))) ? 0.6 : 1
            onClickedPass: {
                if (inSymView) {
                    inSymView2 = !inSymView2
                } else {
                    isShifted = (!isShifted)
                    isShiftLocked = false
                }
            }
            onPressedAndHoldPass: {
                if (!inSymView) {
                    isShifted = true
                    isShiftLocked = true
                }
            }
        }

        Row {
            spacing: keyMargin
            Repeater {
                model: row3
                CharacterKey {
                    width: keyWidth; height: keyHeight
                    caption: row3[index][0]
                    captionShifted: row3[index][0].toUpperCase()
                    symView: row3[index][1]
                    symView2: row3[index][2]
                }
            }
        }

        FunctionKey {
            width: units.gu(UI.PORTRAIT_SHIFT_WIDTH); height: keyHeight
            icon: "icon_backspace.png"
            repeat: true
            onClickedPass: MInputMethodQuick.sendCommit("\b")
        }
    }

    Row { //Row 4
        anchors.horizontalCenter: parent.horizontalCenter
        spacing: (columns == 11) ? 16 : 16
        FunctionKey {
            width: units.gu(UI.PORTRAIT_KEY123_WIDTH); height: keyHeight
            caption: inSymView ? "ABC" : "?123"
            onClickedPass: { inSymView = (!inSymView) }
        }

        Row {
            spacing: 8
            CharacterKey { caption: ","; captionShifted: ","; width: units.gu(UI.portraitWidth); height: keyHeight; }
            CharacterKey { caption: " "; captionShifted: " "; width: units.gu(UI.PORTRAIT_SPACE_WIDTH); height: keyHeight; }
            CharacterKey { caption: "."; captionShifted: "."; width: units.gu(UI.portraitWidth); height: keyHeight; }
        }

        FunctionKey {
            width: units.gu(UI.PORTRAIT_KEY123_WIDTH); height: keyHeight
            icon: "icon_enter.png"
            repeat: true
            caption: MInputMethodQuick.actionKeyOverride.label
            onClickedPass: MInputMethodQuick.activateActionKey()
        }
    }
}
